#include "audiotag.h"
#include <alsa/asoundlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/select.h>
#include <sndfile.h>

static int alsa_process(int n, void *_p)
{
  player                       *p = _p;
  float                        *out = p->outbuf;
  int                          i;
  float                        *in = p->buf;
  int                          start_pos = p->buf_next;
  char                         done;

  for (i = 0; i < n; i++) out[i] = in[(start_pos+i) & 32767];

  p->buf_next = (start_pos+n) & 32767;

  ADD(p->current, n);
  ADD(p->free_samples, n);

  done = 1;
  write(jack_process_write, &done, 1);

  return 0;
}

#define A do { if (err < 0) { fprintf(stderr, "%s:%d: alsa error '%s'\n", __FILE__, __LINE__, snd_strerror(err)); abort(); } } while (0)

static snd_pcm_t *h;

void open_alsa(int samplerate)
{
  snd_pcm_hw_params_t *hw_params;
  snd_pcm_sw_params_t *sw_params;
  snd_pcm_uframes_t   period_frames;
  snd_pcm_uframes_t   buffer_frames;
  unsigned int        rate;
  snd_pcm_format_t    format;
  int err;

  format = SND_PCM_FORMAT_FLOAT_LE;
  err = snd_pcm_open(&h, "default", SND_PCM_STREAM_PLAYBACK, 0); A;
  snd_pcm_hw_params_alloca(&hw_params);
  snd_pcm_sw_params_alloca(&sw_params);
  err = snd_pcm_hw_params_any(h, hw_params); A;
  err = snd_pcm_hw_params_set_access(h, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED); A;
  err = snd_pcm_hw_params_set_format(h, hw_params, format); A;
  err = snd_pcm_hw_params_set_channels(h, hw_params, 1); A;
  rate = samplerate;
  err = snd_pcm_hw_params_set_rate_near(h, hw_params, &rate, 0); A;
  if (rate != samplerate) {printf("bad rate\n"); abort();}
  period_frames = 1024;
  buffer_frames = 1024*3;
  err = snd_pcm_hw_params_set_period_size_near(h,hw_params,&period_frames,0);A;
  err = snd_pcm_hw_params_set_buffer_size_near(h, hw_params, &buffer_frames);A;
  err = snd_pcm_hw_params(h, hw_params); A;
}

void close_alsa(void)
{
  snd_pcm_close(h);
}

void xrun(snd_pcm_t *h)
{
  snd_pcm_prepare(h);
}

void suspend(snd_pcm_t *h)
{
  int res;
  int loop = 0;
  while ((res = snd_pcm_resume(h)) == -EAGAIN && loop < 100) {
    struct timeval t;
    t.tv_sec = 0;
    t.tv_usec = 10 * 1000;
    select(0, 0, 0, 0, &t);
    loop++;
  }
  if (res < 0) snd_pcm_prepare(h);
}

void put_alsa(player *p, int frames)
{
#define X do { fprintf(stderr, "%s:%s:%d: alsa error\n", __FILE__, __FUNCTION__, __LINE__); xrun(h); } while (0)
#define Y do { fprintf(stderr, "%s:%s:%d: alsa error\n", __FILE__, __FUNCTION__, __LINE__); suspend(h); } while (0)
  ssize_t r;
  unsigned char *bb;
  size_t count = frames;

  bb = (unsigned char *)p->outbuf;

  while (count) {
    r = snd_pcm_writei(h, bb, count);
    if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
      /* nothing */
    }
    else if (r == -EPIPE) { usleep(50*1000); X; }
    else if (r == -ESTRPIPE) Y;
    else if (r < 0) abort();
    if (r > 0) {
      count -= r;
      bb += r * 2;
    }
  }
#undef X
#undef Y
}

static void *alsa_thread(void *_p)
{
  player *p = _p;
  char pc;

next:
  while (1) {
    if (read(alsa_read, &pc, 1) != 1) abort();
    if (pc == 1) break;
  }

  open_alsa(p->samplerate);

  while (GET(p->in_play)) {
    alsa_process(1024, p);
    put_alsa(p, 1024);
  }

  close_alsa();
  goto next;
}

void init_alsa(player *pl)
{
  pthread_attr_t attr;
  pthread_t      t;

  pl->in_play = 0;
  if (pl->no_jack) return;

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&t, &attr, alsa_thread, pl)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&t, &attr, audio_thread, pl)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  pl->jack_error = 0;
}
