#include "audiotag.h"
#include <unistd.h>
#include <stdlib.h>

volatile int hi_turn;
volatile int lo_turn;

int gui_read, gui_write;
int slow_compute_lo_read, slow_compute_lo_write;
int fast_compute_lo_read, fast_compute_lo_write;
int slow_compute_hi_read, slow_compute_hi_write;
int fast_compute_hi_read, fast_compute_hi_write;
int jack_player_read, jack_player_write;
int jack_process_read, jack_process_write;
int compute_lo_timer_read, compute_lo_timer_write;
int alsa_read, alsa_write;

void com_compute_hi_fast(int do_slow)
{
  char c;
  ADD(hi_turn, 1);
  c = do_slow+1; if (write(fast_compute_hi_write, &c, 1) != 1) abort();
}

void com_compute_lo_fast(int do_slow)
{
  char c;
  ADD(lo_turn, 1);
  c = do_slow+1; if (write(fast_compute_lo_write, &c, 1) != 1) abort();
}

void com_compute_hi_slow(int do_fast)
{
  char c;
  ADD(hi_turn, 1);
  c = do_fast+1; if (write(slow_compute_hi_write, &c, 1) != 1) abort();
}

void com_compute_lo_slow(int do_fast)
{
  char c;
  ADD(lo_turn, 1);
  c = do_fast+1; if (write(slow_compute_lo_write, &c, 1) != 1) abort();
  /* start timer if do_fast */
  if (do_fast) {
    c = 1; if (write(compute_lo_timer_write, &c, 1) != 1) abort();
  }
}

void com_compute_lo_responsive(void)
{
  com_compute_lo_slow(1);
}

void com_jack_player_started(void)
{
  com_compute_lo_responsive();
#if 0
  char c;
  c = 1; if (write(gui_write, &c, 1) != 1) abort();
  c = 1; if (write(slow_compute_lo_write, &c, 1) != 1) abort();
#endif
}

void com_jack_player_advance(void)
{
  char c;
  c = 0; if (write(gui_write, &c, 1) != 1) abort();
}

void com_jack_player_ended(void)
{
  com_compute_lo_responsive();
#if 0
  char c;
  c = 1; if (write(gui_write, &c, 1) != 1) abort();
  c = 1; if (write(slow_compute_lo_write, &c, 1) != 1) abort();
#endif
}

void com_jack_player_start(void)
{
  char c;
  c = 1; if (write(jack_player_write, &c, 1) != 1) abort();
}

void com_jack_player_stop(void)
{
  char c;
  c = 0; if (write(jack_player_write, &c, 1) != 1) abort();
}

void com_chunk_redraw(void)
{
  com_compute_hi_fast(0);
  com_compute_lo_fast(0);
}

void com_chunk_done(void)
{
#if 0
  /* old code - does not work (if slow is faster than fast, fast will
   * end after and we get crap)
   */
  com_compute_hi_fast(0);
  com_compute_lo_fast(0);
  com_compute_hi_slow(0);
  com_compute_lo_slow(0);
#endif
  /* I hope this one is okay */
  com_compute_hi_slow(1);
  com_compute_lo_slow(1);
}

void com_redraw_gui_now(void)
{
  char c;
  c = 1; if (write(gui_write, &c, 1) != 1) abort();
}

void init_com(void)
{
  int p[2];
  if (pipe(p)) abort();
  gui_read = p[0]; gui_write = p[1];
  if (pipe(p)) abort();
  slow_compute_lo_read = p[0]; slow_compute_lo_write = p[1];
  if (pipe(p)) abort();
  fast_compute_lo_read = p[0]; fast_compute_lo_write = p[1];
  if (pipe(p)) abort();
  slow_compute_hi_read = p[0]; slow_compute_hi_write = p[1];
  if (pipe(p)) abort();
  fast_compute_hi_read = p[0]; fast_compute_hi_write = p[1];
  if (pipe(p)) abort();
  jack_player_read = p[0]; jack_player_write = p[1];
  if (pipe(p)) abort();
  jack_process_read = p[0]; jack_process_write = p[1];
  if (pipe(p)) abort();
  compute_lo_timer_read = p[0]; compute_lo_timer_write = p[1];
  if (pipe(p)) abort();
  alsa_read = p[0]; alsa_write = p[1];
}
